.LSTOUT -
;------------------------------------------------------------------------------
;
; File      : serflash.inc
; Purpose   : Macros  for Atmel AT45DBxxx Serial Flash access
; Project   : DK Library
; Assembler : mca-se
;
; Copyright (c) 2003 by Sensory Inc., All Rights Reserved
;
;------------------------------------------------------------------------------
;13-apr-04   rp  - added stream read macros
;18-aug-2003 ea  - port from 24xx serial EEPROM driver
;------------------------------------------------------------------------------
;
; NOTE: These macros provide random access to the Atmel AT45DBxxx memory
;       These are the "specific" macros for this device,
;       as opposed to the "generic" macros in xmem.inc.
;       Specific macros are only provided for those functions
;       that actually exist for the device ( not for stubs ).
;
;------------------------------------------------------------------------------

SFLASHDEFS_INCLUDED .EQU 1

;------------------------------------------------------------------------------

        .EXTRN (code) FnSerFlashDisable
        .EXTRN (code) FnSerFlashEnable
        .EXTRN (code) FnSerFlashGetSize
        .EXTRN (code) FnSerFlashReadByte
        .EXTRN (code) FnSerFlashReadStart
        .EXTRN (code) FnSerFlashReadNext
        .EXTRN (code) FnSerFlashReadFinish
        .EXTRN (code) FnSerFlashReady
        .EXTRN (code) FnSerFlashWriteByte
        .EXTRN (code) FnSerFlashWriteNextByte
        .EXTRN (code) FnSerFlashBlockErase
        .EXTRN (code) FnSerFlashWriteBuf1Byte
        .EXTRN (code) FnSerFlashWriteNextBuf1Byte
        .EXTRN (code) FnSerFlashWriteNextBuf1ByteDone
        .EXTRN (code) FnSerFlashBuf1ToMain

;-----------------------------------------------------------------------------
; SFlashDisable - Disable the device for reading and writing
;
; in:
; out:
;
; NOTE: This routine undoes the effect of SFlashEnable.
;       Call once when done invoking other random access routines
;
SFlashDisable .MACRO
        call    FnSerFlashDisable
        .ENDMAC


;-----------------------------------------------------------------------------
; SFlashEnable - Enable the device for random reading and writing
;
; in:
; out:
;
; NOTE: This routine could be called multiple times in an application,
;       as opposed to SFlashInitialize.
; NOTE: Call once before invoking other random access routines
;
SFlashEnable .MACRO
        call    FnSerFlashEnable
        .ENDMAC


;-----------------------------------------------------------------------------
; SFlashGetSize - Get size of device
;
; in:
; out:  r0:r2 = Size (also last address)
;       dptr  = Number of pages (if applicable for the device, else 0)
;

SFlashGetSize .MACRO
        call    FnSerFlashGetSize
        .ENDMAC

;------------------------------------------------------------------------------
; SFlashReadByte - Read one byte from external memory
;
; in:   %addr = 24-bit address of byte to read
; out:  a = value read
;       xptr points past address read
;
;
SFlashReadByte .MACRO  addr
        movT    xptr, addr
        call    FnSerFlashReadByte
        .ENDMAC

;------------------------------------------------------------------------------
; SFlashReadStart - Read one byte from external memory stream
;
; in:   %addr = 24-bit address of byte to read
; out:  a = value read
;       xptr points past address read
;
;
SFlashReadStart .MACRO  addr
        movT    xptr, addr
        call    FnSerFlashReadStart
        .ENDMAC

;------------------------------------------------------------------------------
; SFlashReadNextByte - Read one byte from external memory
;                    following initial call to SFlashReadByte
;
; in:   xptr = 24-bit address of byte to read
;               ( setup by previous call to SFlashReadByte )
; out:  a = value read
;       xptr points past address read
;
SFlashReadNextByte .MACRO
        call    FnSerFlashReadNext
        .ENDMAC

;------------------------------------------------------------------------------
; SFlashReadFinish - Complete stream read
;
; in:   
; out:  
;

SFlashReadFinish .MACRO  
        call    FnSerFlashReadFinish
        .ENDMAC

;------------------------------------------------------------------------------
; SFlashReady - Test for device ready to accept any command
;
; in:
; out:  carry clear if ready, else set
;
; NOTE: Waits for IIC bus to become ready
;
SFlashReady .MACRO
        call    FnSerFlashReady
        .ENDMAC

;-----------------------------------------------------------------------------
; SFlashWriteByte - Write one byte to external memory
;
; in:   xptr = 24-bit address of byte to write
;       %val = value to write
; out:  xptr points past address written
; uses: a = value written
;
SFlashWriteByte .MACRO  addr, val
        movT    xptr, addr
        mov     a, val
        call    FnSerFlashWriteByte
        .ENDMAC

;-----------------------------------------------------------------------------
; SFlashWriteNextByte - Write one byte to external memory
;                     following initial call to SFlashWriteByte
;
; in:   %val = value to write
;       xptr = 24-bit address of byte to write
;               (setup by previous call to SFlashWriteByte)
; out:  xptr points past address written
; uses: a = value written
;
SFlashWriteNextByte .MACRO  val
        mov     a, val
        call    FnSerFlashWriteByte
        .ENDMAC
;-----------------------------------------------------------------------------





;-----------------------------------------------------------------------------
; SFlashWriteBuf1Byte - Write into buffer 1
;
; in:   %val = value to write
;       xptr0 = 8-bit address of byte to write
;               (setup by previous call to SFlashWriteByte)
;

SFlashWriteBuf1Byte .MACRO  addr, val
        mov     xptr0, addr
        mov     a, val
        call    FnSerFlashWriteBuf1Byte
        .ENDMAC

SFlashWriteNextBuf1Byte .MACRO  val
        mov     a, val
        call    FnSerFlashWriteNextBuf1Byte
        .ENDMAC

SFlashWriteNextBuf1ByteDone .MACRO
        call    FnSerFlashWriteNextBuf1ByteDone
        .ENDMAC

;-----------------------------------------------------------------------------
; FnSerFlashBuf1ToMain - Write buffer 1 into the main memory
;
; in:   xptr = 24-bit address of byte to write
;
SFlashBuf1ToMain .MACRO  addr
        movT    xptr, addr
        call    FnSerFlashBuf1ToMain
        .ENDMAC

;-----------------------------------------------------------------------------

.LSTOUT .

