Attribute VB_Name = "SensoryLoader"
' Downloader GUI for the RSC4128 Demo/Eval Board and VR Stamp
'26-apr-06  rp  v1.6.2  - support drag & drop to desktop icon -- filenames are in quotes.
'                         support drag & drop from explorer to open GUI
'                         increase debug watch window to 1000 lines
'                         restored focus to download button whenever it is enabled.
'25-jan-06  rp  v1.6.0  - modified for DE2: support 4Mbit-32Mbit serial data flash
'                         using new command for v4.02 or higher to read serial flash density.
'                         Also, changed Debug Output code to capture first line if not prefix CRLF
'31-oct-05  rp  v1.5.2  - increased receive buffer size to 4096 bytes.
'                         tested for chars in buffer at head of frmWatchDebug
'12-oct-05  rp  v1.5.1  - restored minimize control box which disappeared mysteriously
'03-oct-05  rp  v1.5.0  - Added debug output parser
'29-sep-05  rp  v1.4.0  - Added microphone test module
'14-sep-05  rp  v1.3.1  - for VR stamp, added warning messages for
'                         baud rates above 57600 and better prompt
'23-aug-05  rp  v1.3.0  - added support for Demo/Eval 2:
'                           command to switch to 230400 baud using
'                           "SetBaudRate".  Added commands to download/upload
'                           to SRAM only for extra speed using secret Alt-0
'                           key code.  Added "constant" mode to download
'                           buffer command(s) for DE2 only.
'08-mar-05  rp  v1.2.3  - wait for extra ACK at end of GetBuffer,
'                         restore display of version number in caption
'31-jan-05  rp  v1.2.2  - enumerate com ports between com5 and com16
'24-jan-05  rp  v1.2.1  - changed caption of minimized form to display file name,
'                         changed stop bits from 2 to 1
'03-jan-05  rp  v1.2.0  - use byte array to transmit serially rather than string
'10-nov-04  rp  v1.1.0  - added support for data space
'(C) 2004 by Sensory, Inc.

Option Explicit

Private Declare Function GetShortPathName Lib "kernel32" Alias "GetShortPathNameA" (ByVal lpszLongPath As String, ByVal lpszShortPath As String, ByVal lBuffer As Long) As Long
Declare Function PlaySound Lib "winmm.dll" Alias "PlaySoundA" (ByVal lpszName As String, ByVal hModule As Long, ByVal dwFlags As Long) As Long
Public Const SND_ALIAS = &H10000     '  name is a WIN.INI [sounds] entry

Public Const IN_BUFFER_SIZE = 4096

Public Const BOOTSECTOR_START_8MBIT = &HF0000
Public Const BOOTSECTOR_START_4MBIT = &H70000
Public Const DEVICE_ID_4MBIT = &HB9
Public Const DEVICE_ID_8MBIT = &HDA

' version 5+ device id has manufacturer*256+device code

Public Const DEVICE_ID_AMD1MBIT = &H16E&
Public Const DEVICE_ID_SST512KBIT = &HBFD4&
Public Const DEVICE_ID_SST1MBIT = &HBFD5&
Public Const DEVICE_ID_SST2MBIT = &HBFD6&
Public Const DEVICE_ID_SST4MBIT = &HBFD7&

Public Const CMD_ACK = 0              ' request an ACK ("A")
Public Const CMD_VERSION = 1          ' request rsc4 software version information
Public Const CMD_UPLOADDATA = 2       ' transmit RSC data flash ==> HOST
Public Const CMD_DNLOADDATA = 3       ' transmit HOST ==> RSC data flash
Public Const CMD_UPLOADCODE = 4       ' transmit RSC code flash ==> HOST
Public Const CMD_DNLOADCODE = 5       ' transmit HOST ==> RSC code flash
Public Const CMD_ERASECFSECTOR = 6    ' erase code flash sector
Public Const CMD_UPLOADSTICK = 7      ' transmit RSC stick ==> HOST
Public Const CMD_DNLOADSTICK = 8      ' transmit HOST ==> RSC stick
Public Const CMD_ERASECODECHIP = 10   ' erase code flash chip
Public Const CMD_GETDEVICEID = 11     ' get id of code flash chip
Public Const CMD_ERASEDATACHIP = 12   ' erase data flash chip
Public Const CMD_CHANGEBAUD = 13      ' change baud rate on the fly
Public Const CMD_UPLOADCODESRAM = 14  ' transmit RSC code flash ==> HOST
Public Const CMD_DNLOADCODESRAM = 15  ' transmit HOST ==> RSC code flash
Public Const CMD_GETSFLASHDENSITY = 16 ' transmit 4-bit AT45xxx density code
Public Const CMD_GETIDPFMFG = 17       ' get manufacturer and product id

Public Const CODE_ACK = &H41        ' "A"

Public Const MAX_BUFFER_SIZE = &H400000   ' 4 megabyte

Public Const MAX_TERMINAL_LINES = 1024
Public Const MAX_LINE_LENGTH = 1024

Public portNumber As Integer        ' 1 = COM1 etc.
Public baudRate As Long             ' 9600 - 115200
Public timerCount As Long           ' down counter used byte timer1 (10 hz)
Public timerDuration As Long        ' reload value for timerCount
Public versionHigh As Integer
Public versionLow As Integer
Public Data(MAX_BUFFER_SIZE) As Byte        ' 1 megabyte buffer
Public cancel As Boolean
Public browseDir As String
Public TerminalActive As Boolean
Public TerminalLines(0 To MAX_TERMINAL_LINES - 1) As String
Public OldestLine As Integer         ' index into TerminalLines circular buffer
Public NewestLine As Integer         ' index into TerminalLines circular buffer
Public FirstLine As Integer          ' index into TerminalLines circular buffer,
                                     ' line at top of display
Public NumLines As Integer           ' number of lines displayed in text box
Public CursorLine As Integer
Public CursorColumn As Integer

Public TS As String

' Open the communications device specified by port number and baud rate.
' Return TRUE if successful,  FALSE if not.
'

Public Function OpenPort(ByVal portNumber As Integer, ByVal baudRate As Long) As Boolean
    OpenPort = False                ' assume failure
    If baudRate > 115200 Then baudRate = 115200
    On Error GoTo ErrorExit
    With Form1.MSComm1
        
        If .PortOpen Then Exit Function  ' fail if already open
        .CommPort = portNumber
        .SThreshold = 0
        .RThreshold = 0
        .Settings = CStr(baudRate) + ",N,8,1"
        .RTSEnable = False
        .DTREnable = False
        .InputMode = comInputModeBinary   ' binary mode
        .InBufferSize = IN_BUFFER_SIZE
        .InBufferCount = 0
        
        .PortOpen = True

        OpenPort = .PortOpen    ' return status is TRUE if we opened the port
        .InBufferCount = 0      ' flush input buffer
    End With
    Exit Function
ErrorExit:
    Debug.Print Err, Err.Description, Err.Source
    OpenPort = False
End Function

Public Sub ClosePort()
    If Form1.MSComm1.PortOpen Then
        Form1.MSComm1.PortOpen = False
    End If
End Sub


Public Function SendCommandAck(buffer() As Byte) As Boolean
    Dim retries As Integer
    retries = 2
    Do While retries > 0
        Form1.MSComm1.InBufferCount = 0
        Form1.MSComm1.Output = buffer
        If GetByte = CODE_ACK Then
            SendCommandAck = True
            Exit Function
        End If
        retries = retries - 1
    Loop
    SendCommandAck = False
End Function

' Transmit one or more bytes in a byte array

Public Sub Send(buffer() As Byte)
    Form1.MSComm1.Output = buffer
End Sub

' Transmit one byte

Public Sub SendByte(b As Byte)
    Dim buffer(0 To 0) As Byte
    buffer(0) = b
    Form1.MSComm1.Output = buffer
End Sub


' Receive one byte and return it as an integer, 0 to 255.
' If timed out then return -1

Public Function GetByte() As Integer
    Dim buffer() As Byte
    With Form1.MSComm1
        timerCount = timerDuration         ' approx 1 second timeout
        Do While timerCount > 0
            If .InBufferCount >= 1024 Then .InBufferCount = 0 ' gone crazy!
            If .InBufferCount > 0 Then
                .InputLen = 1   ' read just 1 byte
                buffer = .Input
                GetByte = buffer(0)
                Exit Function
            End If
            DoEvents
        Loop
    End With
    GetByte = -1        ' timed out
End Function

Public Sub Flush()
    Form1.MSComm1.InBufferCount = 0
End Sub

' Receive N bytes and store in a variable length byte array.
' Returns the number of bytes read, 0 if error.

Public Function GetBytes(length As Integer, buffer() As Byte) As Integer
    GetBytes = 0
    ReDim buffer(0 To length - 1)
    With Form1.MSComm1
        timerCount = 10         ' approx 1 second timeout
        Do While timerCount > 0
            If .InBufferCount >= length Then
                .InputLen = 0
                buffer = .Input
                GetBytes = length
                Exit Function
            End If
            DoEvents
        Loop
    End With
End Function



'
' return TRUE if the XOR sum of N bytes is zero
'

Public Function Checksum(s() As Byte, n As Integer) As Boolean
    Dim i As Integer
    Dim c As Byte
    
    Checksum = False
    c = 0
    For i = 0 To n - 1
        c = c Xor s(i)
    Next i
    If c = 0 Then Checksum = True
End Function

Public Sub AddChecksum(s() As Byte, start As Integer, last As Integer)
    Dim i As Integer
    Dim csum As Byte
    
    csum = 0
    ' checksum does not include command byte (why?)
    For i = start To last
        csum = csum Xor s(i)
    Next i
    s(last + 1) = csum
End Sub

' Convert a long path to a short 8.3 path
Function GetShortPath(ByVal LongPath As String) As String
    
    Dim s As String
    Dim i, PathLength As Long
        
    s = Space(255)
    i = Len(s)

    PathLength = GetShortPathName(LongPath, s, i)
    GetShortPath = Left$(s, PathLength)

End Function


'substitute the split() function
Public Function NeatSplit(ByVal Expression As String, _
    Optional ByVal Delimiter As String = " ", _
    Optional ByVal Limit As Long = -1, _
    Optional Compare As VbCompareMethod = vbBinaryCompare) As Variant

    Dim varItems As Variant, i As Long

    varItems = Split(Expression, Delimiter, Limit, Compare)

    For i = LBound(varItems) To UBound(varItems)
        If Len(varItems(i)) = 0 Then varItems(i) = Delimiter
    Next i

    NeatSplit = Filter(varItems, Delimiter, False)

End Function


'return the requested part of the full path also in short version
Function SplitPath(ByVal fpath As String, Mask As String, short As Boolean) As String
    
    Dim retpath As String
    Dim retname As String
    Dim retext As String
    
    If fpath = "" Then
        SplitPath = ""
        Exit Function
    End If
    
    'get short version
    If short Then fpath = GetShortPath(fpath)
        
    'get the path
    retpath = Left(fpath, InStrRev(fpath, "\") - 1)
    
    'get the name and extention
    retname = Right(fpath, (Len(fpath) - InStrRev(fpath, "\", -1)))
    retext = Right(retname, (Len(retname) - InStrRev(retname, ".", -1)))
    retname = Left(retname, (InStrRev(retname, ".") - 1))
 
    'set return value
    Select Case Mask
        Case "001" 'ext
            SplitPath = retext
        Case "010" 'name
            SplitPath = retname
        Case "011" 'name + ext
            SplitPath = retname & "." & retext
        Case "100" 'path
            SplitPath = retpath
        Case "110" 'path + name
            SplitPath = retpath & "\" & retname
        Case "111" 'path + name +ext
            SplitPath = retpath & "\" & retname & "." & retext
    End Select

End Function



Public Sub WriteINIFile(s As String)
    Dim fn As Integer
    
    On Error GoTo ErrorExit
    fn = FreeFile
    Open s For Output As #fn
    
    Print #fn, "port", Form1.cbxPort.ListIndex + 1
    Print #fn, "baud", val(Form1.cbxBaud.Text)
    Print #fn, "filename", Chr(34) & Form1.txtName.Text & Chr(34)
    Print #fn, "filterindex", Form1.dlgMain.FilterIndex
    Print #fn, "browsedir", Chr(34) & browseDir & Chr(34)
    Print #fn, "opthex", Form1.optHex
    Print #fn, "optcodeconst", Form1.optCodeConst
    Print #fn, "chkverify", Form1.chkVerify
    
ErrorExit:
    Close #fn
End Sub


Public Sub ReadINIFile(s As String)
    Dim fn As Integer, i As Integer
    Dim line As String
    Dim tag As String
    Dim v As String
    
    On Error GoTo ErrorExit
    fn = FreeFile
    Open s For Input As #fn
    
    Do While Not EOF(fn)
        Line Input #fn, line
    
        tag = ""
        v = ""
        For i = 1 To Len(line)
            If Mid(line, i, 1) = " " Then Exit For
            tag = tag + Mid(line, i, 1)
        Next i
        If tag = "" Then GoTo ReadNext
        
        Do While i <= Len(line)
            If Mid(line, i, 1) <> " " Then Exit Do
            i = i + 1
        Loop
        If i > Len(line) Then GoTo ReadNext
        v = Mid(line, i)
        Do
            If Right(v, 1) = " " Then
                v = Left(v, Len(v) - 1)
            Else
                Exit Do
            End If
        Loop
        
        If Left(v, 1) = Chr(34) Then
            v = Mid(v, 2)
        End If
        If Right(v, 1) = Chr(34) Then
            v = Left(v, Len(v) - 1)
        End If
                       
     
        If tag = "port" Then
            Form1.cbxPort.ListIndex = CInt(v) - 1
        ElseIf tag = "baud" Then
            For i = 0 To Form1.cbxBaud.ListCount - 1
                If val(v) = val(Form1.cbxBaud.List(i)) Then
                    Form1.cbxBaud.Text = Form1.cbxBaud.List(i)
                    Exit For
                End If
            Next i
            
        ElseIf tag = "filename" Then
            Form1.txtName.Text = v
        ElseIf tag = "filterindex" Then
            Form1.dlgMain.FilterIndex = CInt(v)
        ElseIf tag = "browsedir" Then
            browseDir = v
        ElseIf tag = "opthex" Then
            If LCase(v) = "true" Then
                Form1.optBinary = False
                Form1.optHex = True
            Else
                Form1.optBinary = True
                Form1.optHex = False
            End If
            
        ElseIf tag = "optcodeconst" Then
          If LCase(v) = "true" Then
                Form1.optData = False
                Form1.optCodeConst = True
            Else
                Form1.optData = True
                Form1.optCodeConst = False
            End If
        ElseIf tag = "chkverify" Then
            Form1.chkVerify = CInt(v)
        End If
        
        
ReadNext:
    
    Loop
        
'    Print #fn, "port", Form1.cbxPort.ListIndex + 1
'    Print #fn, "baud", Form1.cbxBaud.Text
'    Print #fn, "filename", Chr(34) & Form1.txtName & Chr(34)
'    Print #fn, "filterindex", Form1.dlgMain.FilterIndex
'    Print #fn, "browsedir", Chr(34) & browseDir & Chr(34)
'    Print #fn, "opthex", Form1.optHex
'    Print #fn, "optcodeconst", Form1.optCodeConst
'    Print #fn, "chkverify", Form1.chkVerify
    
ErrorExit:
    Close #fn
End Sub
